source ROOTWINDOW.tcl
source SLFCONSTS.tcl
source MATH.tcl
source SLIDEUI.tcl

set SLF_STEREO_MONO 0
set SLF_STEREO_RED_GREEN 1
set SLF_STEREO_CROSS 2
set SLF_STEREO_DIVERGE 3
set SLF_STEREO_HMD 4

proc CreateStereoTypeFrame { name parent } {
    set root $parent

    set stereoTypeList [list SLF_STEREO_MONO SLF_STEREO_RED_GREEN SLF_STEREO_CROSS SLF_STEREO_DIVERGE SLF_STEREO_HMD]
    set enumList [CreateSLIDEEnumList $stereoTypeList]

    global SLF_STEREO_MONO

    return [CreateRadioButtonFrame $name $root stereoType "Stereo Type" $SLF_STEREO_MONO $enumList]
}

proc CreateCameraVariables { name } {
    global $name SLF_PERSPECTIVE

    set [subst $name](projection) $SLF_PERSPECTIVE
    set [subst $name](left) -0.5
    set [subst $name](right) 0.5
    set [subst $name](bottom) -0.5
    set [subst $name](top) 0.5
    set [subst $name](back) -2.00
    set [subst $name](front) -0.01
    set [subst $name](mpu) 1.0
}

proc CreateStereoCameraUI { parent name toglwin } {

    if { $parent == {} } {
	set root ".slf_[subst $name]"
    } elseif { $parent == "." } {
	set root ".slf_[subst $name]"
    } else {
	set root "$parent.slf_[subst $name]"
    }

    toplevel $root

    ##################################################
    # Projection

    set projectionFrame [CreateProjectionFrame $name $root]

    ##################################################
    # Stereo Type

    set stereoTypeFrame [CreateStereoTypeFrame $name $root]

    ##################################################
    # Eye Separation

    set eyeSeparationFrame [CreateScale $name $root eyeSeparation "Eye Separation" 0.065 0.010 0.100 0.001 1 horizontal]

    ##################################################
    # Meters Per Pixel

    set metersPerPixelFrame [CreateScale $name $root metersPerPixel "Meters Per Pixel" 0.00028 0.00010 0.00100 0.00001 1 horizontal]

    ##################################################
    # Viewing Distance

    set viewingDistanceFrame [CreateScale $name $root viewingDistance "Viewing Distance" 0.57 0.01 3.00 0.01 1 horizontal]

    ##################################################
    # HMD Meters Per Pixel

    set metersPerPixelHMDFrame [CreateScale $name $root metersPerPixelHMD "HMD Meters Per Pixel" 0.00004 0.00001 0.00100 0.00001 1 horizontal]

    ##################################################
    # HMD Viewing Distance

    set viewingDistanceHMDFrame [CreateScale $name $root viewingDistanceHMD "HMD Viewing Distance" 0.03 0.01 3.00 0.01 1 horizontal]

    ##################################################
    # HMD Viewport Separation

    set vpSeparationHMDFrame [CreateScale $name $root vpSeparationHMD "HMD Viewport Separation" 0.060 0.010 0.100 0.001 1 horizontal]

    ##################################################
    # Clipping

    CreateCameraVariables $name
    CreateCameraVariables "[subst $name]Mono"
    CreateCameraVariables "[subst $name]RedGreenLeft"
    CreateCameraVariables "[subst $name]RedGreenRight"
    CreateCameraVariables "[subst $name]CrossLeft"
    CreateCameraVariables "[subst $name]CrossRight"
    CreateCameraVariables "[subst $name]DivergeLeft"
    CreateCameraVariables "[subst $name]DivergeRight"
    CreateCameraVariables "[subst $name]HMDLeft"
    CreateCameraVariables "[subst $name]HMDRight"

    set backFrontFrame [CreateDoubleScale $name $root backFront back front \
	    Back Front -10 -2 -0.20 -0.01 0.01 1 horizontal]

    pack $projectionFrame $stereoTypeFrame $eyeSeparationFrame $metersPerPixelFrame $viewingDistanceFrame $metersPerPixelHMDFrame $viewingDistanceHMDFrame $backFrontFrame -side top -fill x
}

proc UpdateStereoCamera { parent name toglwin } {

    if { $parent == {} } {
	set root ".slf_[subst $name]"
    } elseif { $parent == "." } {
	set root ".slf_[subst $name]"
    } else {
	set root "$parent.slf_[subst $name]"
    }

    set varStereoType "[subst $name]_stereoType"
    set varEyeSeparation "[subst $name]_eyeSeparation"
    set varViewingDistance "[subst $name]_viewingDistance"
    set varMetersPerPixel "[subst $name]_metersPerPixel"

    set varViewingDistanceHMD "[subst $name]_viewingDistanceHMD"
    set varMetersPerPixelHMD "[subst $name]_metersPerPixelHMD"
    set varVPSeparationHMD "[subst $name]_vpSeparationHMD"

    global $varStereoType
    global $varEyeSeparation $varViewingDistance $varMetersPerPixel
    global $varViewingDistanceHMD $varMetersPerPixelHMD $varVPSeparationHMD

    if { [winfo exists $toglwin] } {
        set winPixelWidth [winfo width $toglwin]
        set winPixelHeight [winfo height $toglwin]
    } else {
	set winPixelWidth [expr 400]
	set winPixelHeight [expr 400]
    }

    # We will use the half width because we will be calculating dimensions
    # from the center of the window, so this is convenient
    set winPixelWidth [expr $winPixelWidth / 2.0]
    set winPixelHeight [expr $winPixelHeight / 2.0]

    set winWidth [expr $winPixelWidth * [subst $$varMetersPerPixel]]
    set winHeight [expr $winPixelHeight * [subst $$varMetersPerPixel]]
    set winWidthHMD [expr $winPixelWidth * [subst $$varMetersPerPixelHMD]]
    set winHeightHMD [expr $winPixelHeight * [subst $$varMetersPerPixelHMD]]

    set eyeOffset [expr [subst $$varEyeSeparation]/2.0]
    set viewingDistance [subst $$varViewingDistance]
    set viewingDistanceHMD [subst $$varViewingDistanceHMD]

    set types { Mono RedGreen Cross Diverge HMD }

    #foreach type $types {

	set type [lindex $types [subst $$varStereoType]]
    	set varType [subst $name][subst $type]

	global $varType

	if { $type == "Mono" } {
	    set virtualVD $viewingDistance

	    set vpWidth [expr $winWidth]
	    set vpHeight [expr $winHeight]
	    
	    set frustumWidth [expr $vpWidth / $virtualVD]
	    set frustumHeight [expr $vpHeight / $virtualVD]
	    set frustumCenterX [expr 0.0]
	    set frustumCenterY [expr 0.0]
	    
	    global $varType
	    set [subst $varType](left) [expr $frustumCenterX - $frustumWidth]
	    set [subst $varType](right) [expr $frustumCenterX + $frustumWidth]
	    set [subst $varType](bottom) [expr $frustumCenterY - $frustumHeight]
	    set [subst $varType](top) [expr $frustumCenterY + $frustumHeight]
	    set [subst $varType](back) $[subst $name](back)
	    set [subst $varType](front) $[subst $name](front)
	    set [subst $varType](mpu) $virtualVD
	} elseif { $type == "RedGreen" } {
	    set virtualVD $viewingDistance

	    set vpWidth [expr $winWidth]
	    set vpHeight [expr $winHeight]
	    
	    set frustumWidth [expr $vpWidth / $virtualVD]
	    set frustumHeight [expr $vpHeight / $virtualVD]
	    set frustumCenterX [expr $eyeOffset / $virtualVD]
	    set frustumCenterY [expr 0.0]
	    
	    global [subst $varType]Left
	    set [subst $varType]Left(left) [expr $frustumCenterX - $frustumWidth]
	    set [subst $varType]Left(right) [expr $frustumCenterX + $frustumWidth]
	    set [subst $varType]Left(bottom) [expr $frustumCenterY - $frustumHeight]
	    set [subst $varType]Left(top) [expr $frustumCenterY + $frustumHeight]
	    set [subst $varType]Left(back) $[subst $name](back)
	    set [subst $varType]Left(front) $[subst $name](front)
	    set [subst $varType]Left(mpu) $virtualVD

	    set frustumCenterX [expr -$frustumCenterX]
	    
	    global [subst $varType]Right
	    set [subst $varType]Right(left) [expr $frustumCenterX - $frustumWidth]
	    set [subst $varType]Right(right) [expr $frustumCenterX + $frustumWidth]
	    set [subst $varType]Right(bottom) [expr $frustumCenterY - $frustumHeight]
	    set [subst $varType]Right(top) [expr $frustumCenterY + $frustumHeight]
	    set [subst $varType]Right(back) $[subst $name](back)
	    set [subst $varType]Right(front) $[subst $name](front)
	    set [subst $varType]Right(mpu) $virtualVD
	} elseif { $type == "Cross" } {
	    set vpWidth [expr $winWidth / 2.0]
	    set vpHeight [expr $winHeight]
	    set vpOffset [expr $vpWidth / 2.0]
	    
	    set virtualVD [expr $eyeOffset * $viewingDistance / ($vpOffset + $eyeOffset)]

	    set frustumWidth [expr $vpWidth / $virtualVD]
	    set frustumHeight [expr $vpHeight / $virtualVD]
	    set frustumCenterX [expr $eyeOffset / $virtualVD]
	    set frustumCenterY [expr 0.0]
	    
	    global [subst $varType]Left
	    set [subst $varType]Left(left) [expr $frustumCenterX - $frustumWidth]
	    set [subst $varType]Left(right) [expr $frustumCenterX + $frustumWidth]
	    set [subst $varType]Left(bottom) [expr $frustumCenterY - $frustumHeight]
	    set [subst $varType]Left(top) [expr $frustumCenterY + $frustumHeight]
	    set [subst $varType]Left(back) $[subst $name](back)
	    set [subst $varType]Left(front) $[subst $name](front)
	    set [subst $varType]Left(mpu) $virtualVD

	    set frustumCenterX [expr -$frustumCenterX]
	    
	    global [subst $varType]Right
	    set [subst $varType]Right(left) [expr $frustumCenterX - $frustumWidth]
	    set [subst $varType]Right(right) [expr $frustumCenterX + $frustumWidth]
	    set [subst $varType]Right(bottom) [expr $frustumCenterY - $frustumHeight]
	    set [subst $varType]Right(top) [expr $frustumCenterY + $frustumHeight]
	    set [subst $varType]Right(back) $[subst $name](back)
	    set [subst $varType]Right(front) $[subst $name](front)
	    set [subst $varType]Right(mpu) $virtualVD
	} elseif { $type == "Diverge" } {
	    set vpWidth [expr $winWidth / 2.0]
	    set vpHeight [expr $winHeight]
	    set vpOffset [expr $vpWidth / 2.0]
	    
	    set virtualVD [expr $eyeOffset * $viewingDistance / $vpOffset]

	    set frustumWidth [expr $vpWidth / $virtualVD]
	    set frustumHeight [expr $vpHeight / $virtualVD]
	    set frustumCenterX [expr $eyeOffset / $virtualVD]
	    set frustumCenterY [expr 0.0]
	    
	    global [subst $varType]Left
	    set [subst $varType]Left(left) [expr $frustumCenterX - $frustumWidth]
	    set [subst $varType]Left(right) [expr $frustumCenterX + $frustumWidth]
	    set [subst $varType]Left(bottom) [expr $frustumCenterY - $frustumHeight]
	    set [subst $varType]Left(top) [expr $frustumCenterY + $frustumHeight]
	    set [subst $varType]Left(back) $[subst $name](back)
	    set [subst $varType]Left(front) $[subst $name](front)
	    set [subst $varType]Left(mpu) $virtualVD

	    set frustumCenterX [expr -$frustumCenterX]
	    
	    global [subst $varType]Right
	    set [subst $varType]Right(left) [expr $frustumCenterX - $frustumWidth]
	    set [subst $varType]Right(right) [expr $frustumCenterX + $frustumWidth]
	    set [subst $varType]Right(bottom) [expr $frustumCenterY - $frustumHeight]
	    set [subst $varType]Right(top) [expr $frustumCenterY + $frustumHeight]
	    set [subst $varType]Right(back) $[subst $name](back)
	    set [subst $varType]Right(front) $[subst $name](front)
	    set [subst $varType]Right(mpu) $virtualVD
	} elseif { $type == "HMD" } {
	    set vpWidth [expr $winWidthHMD]
	    set vpHeight [expr $winHeightHMD]
	    set vpOffset [subst $$varVPSeparationHMD]
	    
	    set virtualVD [expr $eyeOffset * $viewingDistanceHMD / $vpOffset]

	    set frustumWidth [expr $vpWidth / $virtualVD]
	    set frustumHeight [expr $vpHeight / $virtualVD]
	    set frustumCenterX [expr $eyeOffset / $virtualVD]
	    set frustumCenterY [expr 0.0]
	    
	    global [subst $varType]Left
	    set [subst $varType]Left(left) [expr $frustumCenterX - $frustumWidth]
	    set [subst $varType]Left(right) [expr $frustumCenterX + $frustumWidth]
	    set [subst $varType]Left(bottom) [expr $frustumCenterY - $frustumHeight]
	    set [subst $varType]Left(top) [expr $frustumCenterY + $frustumHeight]
	    set [subst $varType]Left(back) $[subst $name](back)
	    set [subst $varType]Left(front) $[subst $name](front)
	    set [subst $varType]Left(mpu) $virtualVD

	    set frustumCenterX [expr -$frustumCenterX]
	    
	    global [subst $varType]Right
	    set [subst $varType]Right(left) [expr $frustumCenterX - $frustumWidth]
	    set [subst $varType]Right(right) [expr $frustumCenterX + $frustumWidth]
	    set [subst $varType]Right(bottom) [expr $frustumCenterY - $frustumHeight]
	    set [subst $varType]Right(top) [expr $frustumCenterY + $frustumHeight]
	    set [subst $varType]Right(back) $[subst $name](back)
	    set [subst $varType]Right(front) $[subst $name](front)
	    set [subst $varType]Right(mpu) $virtualVD
	}

#    }
}





