#ifndef __PARSER_H__
#define __PARSER_H__

#include "scanner.h"

//////////////////////////////////////////////////////////////////////////////
// CParser Class
//

typedef int (YY_PARSE_TYPE) (void);

class CParser
{
public:
  CParser();
  ~CParser();
  VOID Init(CHAR *pcFile, CHAR **ppcDirectories, UINT_32 uiDirectories, 
	    CParser **ppyy_CParser, 
	    YYLTYPE *pyylloc, YY_PARSE_TYPE *pfyy_Parse,
	    CScanner **ppyy_CScanner,
	    int *pyyleng,
	    YY_BEGIN_INITIAL_TYPE *pfyy_BeginInitial,
	    YY_GET_CURRENT_BUFFER_TYPE *pfyy_GetCurrentBuffer,
	    YY_CREATE_BUFFER_TYPE *pfyy_CreateBuffer,
	    YY_DELETE_BUFFER_TYPE *pfyy_DeleteBuffer,
	    YY_SWITCH_TO_BUFFER_TYPE *pfyy_SwitchToBuffer,
	    YY_SET_VALUE_INT_TYPE *pfyy_SetValueInt,
	    YY_SET_VALUE_FLOAT_TYPE *pfyy_SetValueFloat,
	    YY_SET_VALUE_STRING_TYPE *pfyy_SetValueString);

  VOID Uninit();

public:
  VOID *Parse();

public:
  VOID HandleError(UINT_32 uiLine, UINT_32 uiColumn, CHAR *pcFormat, ...);
  VOID HandleWarning(UINT_32 uiLine, UINT_32 uiColumn, CHAR *pcFormat, ...);

public:
  inline UINT_32 GetErrorCount();
  inline UINT_32 GetWarningCount();

  inline VOID *GetAST();
  inline VOID SetAST(VOID *pvAST);

protected:
  FILE *m_fpError;

  UINT_32 m_uiErrorCount;
  UINT_32 m_uiWarningCount;

  CScanner *m_pCScanner;
  VOID *m_pvAST;

  YYLTYPE *m_pyylloc;
  YY_PARSE_TYPE *m_pfyy_Parse;
};

//////////////////////////////////////////////////
// Inlined Functions
//

inline UINT_32 CParser::GetErrorCount()
{
  return m_uiErrorCount + m_pCScanner->GetErrorCount();
}

inline UINT_32 CParser::GetWarningCount()
{
  return m_uiWarningCount + m_pCScanner->GetWarningCount();
}

inline VOID *CParser::GetAST()
{
  return m_pvAST;
}

inline VOID CParser::SetAST(VOID *pvAST)
{
  m_pvAST = pvAST;
}

#endif // __PARSER_H__









