#ifndef __SLIDEFACE_H__
#define __SLIDEFACE_H__

#include "slideprimitive.h"

class CSLIDEWorld;
class CSLIDEPoint;
class CSLIDESurface;

class CScanConverter;

//////////////////////////////////////////////////////////////////////
// CSLIDEFace Class
//

class CSLIDEFaceIter;

class CSLIDEFace : public CSLIDEPrimitive
{
//////////////////////////////////////////////////
// Friends
//
public:
  friend class CSLIDEFaceIter;

//////////////////////////////////////////////////
// Enumerated Types
//

//////////////////////////////////////////////////
// Initialization
//
public:
  CSLIDEFace();
  virtual ~CSLIDEFace();
  virtual VOID Init(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn, CHAR *pcID, 
		    CListInt *plistPointIDs);
  virtual VOID Uninit();

  virtual BOOL Initialized();

  virtual BOOL CheckSemantics(CSLIDEWorld *pCSLIDEWorld);

  virtual BOOL Valid();

//////////////////////////////////////////////////
// Iteration Functions
//
public:
  virtual CSLIDEPrimitiveIter *AllocateIter();
  virtual VOID FreeIter(CSLIDEPrimitiveIter *pCSLIDEFaceIter);

//////////////////////////////////////////////////
// Geometric Computations
//
public:
  VOID InsertIntoBound(CBound *pCBound);
  virtual VOID Preprocess();
//////////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 8: Deal with dynamic Point normals 
//
  VOID PreprocessAutoNormals();
  VOID CheckAutoNormalsDynamic();
//
// END  : ASSIGNMENT 8: Deal with dynamic Point normals 
//////////////////////////////////////////////////////////////////////////////

protected:
  VOID CheckDynamic();
//////////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 8: Deal with dynamic Point normals 
//
  VOID UpdateState();
// END  : ASSIGNMENT 8: Deal with dynamic Point normals 
//////////////////////////////////////////////////////////////////////////////
  VOID CalculateNormal();
  VOID CalculateCentroid();

//////////////////////////////////////////////////
// Rendering
//
public:
//////////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 8: Deal with Lighting and Shading
//
  virtual VOID Render(const Matrix &mGvp_proj, 
		      const Matrix &mGproj_vrc, const Matrix &mGvrc_proj, 
		      const Matrix &mGvrc_obj, const Matrix &mGobj_vrc, 
		      CBound *pCBoundClip, CListInt *plistLights,
		      CSLIDESurface *pCSLIDESurface,
		      SLIDEShadingType eShading, SLIDELODType eLOD, 
		      SLIDESolidType eSolid);
//
// END  : ASSIGNMENT 8: Deal with Lighting and Shading
//////////////////////////////////////////////////////////////////////////////

protected:
  virtual VOID ModifySurfacePointer(CSLIDESurface* &pCSLIDESurface);
  virtual VOID RenderNormal(const Matrix &mGvp_proj, const Matrix &mGproj_vrc, 
			    const Matrix &mGvrc_obj, const Matrix &mGobj_vrc,
			    const Point &pt, const Vector &vNormal);

//////////////////////////////////////////////////
// Output Functions
//
public:
  virtual VOID WriteSLIDE(FILE *fpOutputFile);

//////////////////////////////////////////////////
// Accessor Functions
//
public:
  inline VOID SetSurfaceID(CHAR *pcSurfaceID);

  virtual UINT_32 GetPointCount();
  virtual inline BOOL GetBounded();
  virtual inline VOID SetBounded(BOOL bBounded);
  virtual inline BOOL GetBoundDynamic();
  virtual inline VOID SetBoundDynamic(BOOL bBoundDynamic);

//////////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 8: Deal with dynamic Point normals 
//
  virtual inline BOOL GetAutoNormals();
  virtual inline VOID SetAutoNormals(BOOL bAutoNormals);
  virtual inline BOOL GetAutoNormalsDynamic();
  virtual inline VOID SetAutoNormalsDynamic(BOOL bAutoNormalsDynamic);
//
// END  : ASSIGNMENT 8: Deal with dynamic Point normals 
//////////////////////////////////////////////////////////////////////////////

  static inline BOOL GetBackFaceCullingEnabled();
  static inline VOID SetBackFaceCullingEnabled(BOOL bBackFaceCullingEnabled);
  static inline BOOL GetClipEnabled();
  static inline VOID SetClipEnabled(BOOL bClipEnabled);
  static inline BOOL GetRenderNormalsEnabled();
  static inline VOID SetRenderNormalsEnabled(BOOL bRenderNormalsEnabled);
  static inline FLOAT GetNormalsRenderLength();
  static inline VOID SetNormalsRenderLength(FLOAT fNormalsRenderLength);

  static inline BOOL GetScanConvertEnabled();
  static inline VOID SetScanConvertEnabled(BOOL bScanConvertEnabled);
  static inline CScanConverter *GetScanConverter();
  static inline VOID SetScanConverter(CScanConverter *pCScanConverter);

//////////////////////////////////////////////////
// Class Variables
//
protected:
  static BOOL m_bBackFaceCullingEnabled;
  static BOOL m_bClipEnabled;
  static BOOL m_bRenderNormalsEnabled;
  static FLOAT m_fNormalsRenderLength;

  static CListInt *m_plistVertices;
  static UINT_32 m_uiCount;

  static BOOL m_bScanConvertEnabled;
  static CScanConverter *m_pCScanConverter;

//////////////////////////////////////////////////
// Member Variables
//
protected:
  CHAR *m_pcSurfaceID;
  CSLIDESurface *m_pCSLIDESurface;
  CListInt *m_plistPointIDs;
  CListInt m_listCSLIDEPoint;

  BOOL m_bBounded;
  BOOL m_bBoundDynamic;

//////////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 8: Deal with dynamic Point normals 
//
  BOOL m_bAutoNormals;
  BOOL m_bAutoNormalsDynamic;
//
// END  : ASSIGNMENT 8: Deal with dynamic Point normals 
//////////////////////////////////////////////////////////////////////////////

  Vector m_vNormal;
  Point m_ptCentroid;
};

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//

inline VOID CSLIDEFace::SetSurfaceID(CHAR *pcSurfaceID)
{
  m_pcSurfaceID = pcSurfaceID;
}

inline BOOL CSLIDEFace::GetBounded()
{
  return m_bBounded;
}

inline VOID CSLIDEFace::SetBounded(BOOL bBounded)
{
  m_bBounded = bBounded;
}

inline BOOL CSLIDEFace::GetBoundDynamic()
{
  return m_bBoundDynamic;
}

inline VOID CSLIDEFace::SetBoundDynamic(BOOL bBoundDynamic)
{
  m_bBoundDynamic = bBoundDynamic;
}

inline BOOL CSLIDEFace::GetAutoNormals()
{
  return m_bAutoNormals;
}

inline VOID CSLIDEFace::SetAutoNormals(BOOL bAutoNormals)
{
  m_bAutoNormals = bAutoNormals;
}

inline BOOL CSLIDEFace::GetAutoNormalsDynamic()
{
  return m_bAutoNormalsDynamic;
}

inline VOID CSLIDEFace::SetAutoNormalsDynamic(BOOL bAutoNormalsDynamic)
{
  m_bAutoNormalsDynamic = bAutoNormalsDynamic;
}

inline BOOL CSLIDEFace::GetBackFaceCullingEnabled()
{
  return m_bBackFaceCullingEnabled;
}

inline VOID CSLIDEFace::SetBackFaceCullingEnabled(BOOL bBackFaceCullingEnabled)
{
  m_bBackFaceCullingEnabled = bBackFaceCullingEnabled;
}

inline BOOL CSLIDEFace::GetClipEnabled()
{
  return m_bClipEnabled;
}

inline VOID CSLIDEFace::SetClipEnabled(BOOL bClipEnabled)
{
  m_bClipEnabled = bClipEnabled;
}

inline BOOL CSLIDEFace::GetRenderNormalsEnabled()
{
  return m_bRenderNormalsEnabled;
}

inline VOID CSLIDEFace::SetRenderNormalsEnabled(BOOL bRenderNormalsEnabled)
{
  m_bRenderNormalsEnabled = bRenderNormalsEnabled;
}

inline FLOAT CSLIDEFace::GetNormalsRenderLength()
{
  return m_fNormalsRenderLength;
}

inline VOID CSLIDEFace::SetNormalsRenderLength(FLOAT fNormalsRenderLength)
{
  m_fNormalsRenderLength = fNormalsRenderLength;
}

inline BOOL CSLIDEFace::GetScanConvertEnabled()
{
  return m_bScanConvertEnabled;
}

inline VOID CSLIDEFace::SetScanConvertEnabled(BOOL bScanConvertEnabled)
{
  m_bScanConvertEnabled = bScanConvertEnabled;
}

inline CScanConverter *CSLIDEFace::GetScanConverter()
{
  return m_pCScanConverter;
}

inline VOID CSLIDEFace::SetScanConverter(CScanConverter *pCScanConverter)
{
  m_pCScanConverter = pCScanConverter;
}

//////////////////////////////////////////////////////////////////////
// CSLIDEFaceIter Class
//

class CSLIDEFaceIter : public CSLIDEPrimitiveIter
{
//////////////////////////////////////////////////
// Initialization
//
public:
  CSLIDEFaceIter();
  virtual ~CSLIDEFaceIter();
  virtual VOID Init(CSLIDEFace *pCSLIDEFace);
  virtual VOID Uninit();

//////////////////////////////////////////////////
// Iteration Functions
//
public:
  virtual CSLIDEPoint *PeekFirstCSLIDEPoint();
  virtual CSLIDEPoint *PeekNextCSLIDEPoint();
  virtual CSLIDEPoint *PeekLastCSLIDEPoint();
  virtual CSLIDEPoint *PeekPrevCSLIDEPoint();

//////////////////////////////////////////////////
// Member Variables
//
protected:
  CListIterInt m_iter;
};

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//


#endif // __SLIDEFACE_H__
