#include "slideinstance.h"

#include "slideworld.h"
#include "slidetransform.h"
#include "slidesurface.h"

//////////////////////////////////////////////////////////////////////
// CSLIDEInstance Class
//

CSLIDEInstance::CSLIDEInstance()
{
  m_pcChildID = NULL;
  m_pCSLIDENode = NULL;
  m_pcSurfaceID = NULL;
  m_pCSLIDESurface = NULL;
}

CSLIDEInstance::~CSLIDEInstance()
{
}

VOID CSLIDEInstance::Init(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn,
			  CHAR *pcChildID)
{
  CSLIDENode::Init(pcFile, uiLine, uiColumn, NULL);

  ASSERT( pcChildID != NULL );

  m_pcChildID = pcChildID;
  m_pCSLIDENode = NULL;
  m_pcSurfaceID = gcppcSLIDEReservedWords[sliderwtSLF_INHERIT];
  m_pCSLIDESurface = NULL;
  m_dyniShading.SetValue(slideshtSLF_INHERIT);
  m_dyniLOD.SetValue(slidelodtSLF_FULL);
  
  m_listCSLIDETransform.Init();
  m_mTransform.SetIdentity();
  m_mInverse.SetIdentity();
}

VOID CSLIDEInstance::Uninit()
{
  CSLIDENode::Uninit();
}

BOOL CSLIDEInstance::Initialized()
{
  if ( !CSLIDENode::Initialized() )
    {
      return FALSE;
    }

  return TRUE;
}

VOID CSLIDEInstance::InsertTransform(CSLIDETransform *pCSLIDETransform)
{
  ASSERT( pCSLIDETransform != NULL );

  m_listCSLIDETransform.InsertLast(pCSLIDETransform, 0);
}

BOOL CSLIDEInstance::CheckSemantics(CSLIDEWorld *pCSLIDEWorld)
{
  if ( !CSLIDENode::CheckSemantics(pCSLIDEWorld) )
    {
      return FALSE;
    }

  m_pCSLIDESurface = pCSLIDEWorld->PeekSurface(m_pcSurfaceID);
  if ( m_pCSLIDESurface == NULL )
    {
      HandleError("Undefined %s \"%s\"", 
		  gcppcSLIDEReservedWords[sliderwtSurface], 
		  m_pcSurfaceID);
      return FALSE;
    }

  ASSERT( 0 <= m_dyniShading.GetValue() &&
	  m_dyniShading.GetValue() < slideshtSize );
  ASSERT( 0 <= m_dyniLOD.GetValue() &&
	  m_dyniLOD.GetValue() < slidelodtSize );

  m_pCSLIDENode = pCSLIDEWorld->PeekNode(m_pcChildID);
  if ( m_pCSLIDENode == NULL )
    {
      HandleError("Undefined %s \"%s\"", 
		  gcppcSLIDEReservedWords[sliderwtObject], m_pcChildID);
      return FALSE;
    }

  return TRUE;
}

BOOL CSLIDEInstance::Valid()
{
  if ( !CSLIDENode::Valid() )
    {
      return FALSE;
    }

  return TRUE;
}

CSLIDEPrimitiveIter *CSLIDEInstance::AllocateIter()
{
  CSLIDEInstanceIter *pCSLIDEInstanceIter;

  pCSLIDEInstanceIter = new CSLIDEInstanceIter;
  ASSERT(pCSLIDEInstanceIter != NULL );
  pCSLIDEInstanceIter->Init(this);

  return pCSLIDEInstanceIter;
}

VOID CSLIDEInstance::FreeIter(CSLIDEPrimitiveIter *pCSLIDEInstanceIter)
{
  delete pCSLIDEInstanceIter;
}

CSLIDECamera *CSLIDEInstance::FindCamera(CListIterInt *piterIDs, 
					 CListInt *plistCSLIDEInstance)
{
  ASSERT( piterIDs != NULL );
  ASSERT( plistCSLIDEInstance != NULL );

  return m_pCSLIDENode->FindCamera(piterIDs, plistCSLIDEInstance);
}

CSLIDELight *CSLIDEInstance::FindLight(CListIterInt *piterIDs, 
				       CListInt *plistCSLIDEInstance)
{
  ASSERT( piterIDs != NULL );
  ASSERT( plistCSLIDEInstance != NULL );

  return m_pCSLIDENode->FindLight(piterIDs, plistCSLIDEInstance);
}

VOID CSLIDEInstance::Preprocess()
{
  UpdateState();
  CheckDynamic();
  SetBoundEmpty(TRUE);
  SetBoundChanged(FALSE);
  SetBoundValid(FALSE);
}

VOID CSLIDEInstance::CheckDynamic()
{
  CListIterInt iter;
  CSLIDETransform *pCSLIDETransform;
  BOOL bTransformDynamic;

  // Check if the NodeID is dynamic
  SetDynamic(m_dyniLOD.GetDynamic() ||
	     ((m_dyniLOD.GetValue() != slidelodtSLF_OFF) &&
	      m_dyniShading.GetDynamic()));

  bTransformDynamic = FALSE;
  iter.Init(&m_listCSLIDETransform);

  for ( pCSLIDETransform = (CSLIDETransform*)iter.PeekFirst(); 
	pCSLIDETransform != NULL;
	pCSLIDETransform = (CSLIDETransform*)iter.PeekNext() )
    {
      if ( pCSLIDETransform->GetDynamic() )
	{ 
	  bTransformDynamic = TRUE;
	  break;
	}
    }

  SetTransformDynamic((!(!m_dyniLOD.GetDynamic() &&
			 (m_dyniLOD.GetValue() == slidelodtSLF_OFF))) &&
		      bTransformDynamic);
}

VOID CSLIDEInstance::PreprocessGraph()
{
  if ( !GetBoundValid() )
    {
      // Check if the Bound is Dynamic
      m_pCSLIDENode->PreprocessGraph();

      SetBounded(m_pCSLIDENode->GetBounded() && 
		 (m_dyniLOD.GetDynamic() || 
		  (m_dyniLOD.GetValue() != slidelodtSLF_OFF)));
      SetBoundDynamic(m_pCSLIDENode->GetBounded() && 
		      (m_dyniLOD.GetDynamic() || 
		       ((m_dyniLOD.GetValue() != slidelodtSLF_OFF) && 
			(GetTransformDynamic() || 
			 m_pCSLIDENode->GetBoundDynamic())))); 
      ASSERT( !(!GetBounded() && GetBoundDynamic()) );
                  
      CalculateBound();
      SetBoundChanged(FALSE);
      SetBoundValid(TRUE);
    }
}

VOID CSLIDEInstance::Update()
{
  SetBoundChanged(FALSE);
  if ( GetDynamic() || GetTransformDynamic() )
    {
      UpdateState();
    }

  if ( GetBoundDynamic() )
    {
      SetBoundValid(FALSE);
    }
}

VOID CSLIDEInstance::UpdateState()
{
  // Shading
  m_dyniShading.Update();
  if ( !( 0 <= m_dyniShading.GetValue() &&
	  m_dyniShading.GetValue() < slideshtSize ) )
    {
      HandleError("Illegal Shading flag value:  %ld", 
		  m_dyniShading.GetValue());
      m_dyniShading.SetValue(slideshtSLF_FLAT);
    }

  // LOD
  m_dyniLOD.Update();
  if ( !( 0 <= m_dyniLOD.GetValue() &&
	  m_dyniLOD.GetValue() < slideshtSize ) )
    {
      HandleError("Illegal LOD flag value:  %ld", 
		  m_dyniLOD.GetValue());
      m_dyniLOD.SetValue(slidelodtSLF_FULL);
    }

  // Smash updated Transform list into a single Matrix

  CListIterInt iter;
  CSLIDETransform *pCSLIDETransform;

  m_mTransform.SetIdentity(); 
  m_mInverse.SetIdentity(); 

  iter.Init(&m_listCSLIDETransform);

  for ( pCSLIDETransform = (CSLIDETransform*)iter.PeekFirst(); 
	pCSLIDETransform != NULL;
	pCSLIDETransform = (CSLIDETransform*)iter.PeekNext() )
    {
      m_mTransform = pCSLIDETransform->GetMatrix() * m_mTransform;      
      m_mInverse = m_mInverse * pCSLIDETransform->GetInverse();
    }
}

VOID CSLIDEInstance::UpdateGraph()
{
  if ( !GetBoundValid() )
    {
      if ( m_dyniLOD.GetValue() == slidelodtSLF_OFF )
	{
	  if ( !GetBoundEmpty() )
	    {
	      SetBoundChanged(TRUE);
	      SetBoundEmpty(TRUE);
	    }
	}
      else
	{
	  m_pCSLIDENode->UpdateGraph();
	  CalculateBound();
	}
      SetBoundValid(TRUE);
    }
}

VOID CSLIDEInstance::InsertIntoBound(CBound &bound)
{
  if ( !GetBoundEmpty() )
    {
      m_pCSLIDENode->InsertIntoBound(m_mTransform, bound);
    }
}

VOID CSLIDEInstance::CalculateBound()
{
  BOOL bBoundEmpty, bBoundChanged;

  // If this was called it means that the group above it
  // in the scene graph must recompute its bounding box

  bBoundEmpty = ((m_dyniLOD.GetValue() == slidelodtSLF_OFF) || 
		 m_pCSLIDENode->GetBoundEmpty());
  bBoundChanged = (((GetBoundEmpty() && !bBoundEmpty) ||
		    (!GetBoundEmpty() && bBoundEmpty)) ||
		   (!m_pCSLIDENode->GetBoundEmpty() && 
		    (m_pCSLIDENode->GetBoundChanged() ||
		     GetTransformDynamic())));

  SetBoundEmpty(bBoundEmpty);
  SetBoundChanged(bBoundChanged);
}

VOID CSLIDEInstance::Render(const Matrix &mGvp_proj, 
			    const Matrix &mGproj_vrc, 
			    const Matrix &mGvrc_proj, 
			    const Matrix &mGvrc_obj, 
			    const Matrix &mGobj_vrc, 
			    CBound *pCBoundClip, 
			    CListInt *plistLights,
			    CSLIDESurface *pCSLIDESurface,
			    SLIDEShadingType eShading, 
			    SLIDELODType eLOD)
{
  ModifySurfacePointer(pCSLIDESurface);
  ModifyShadingFlag(eShading);
  ModifyLODFlag(eLOD);

  if ( eLOD == slidelodtSLF_OFF )
    {
      // This branch has been turned off by the LOD flags
      return;
    }

  // We are not preforming bounding box rejections on instances

  Matrix mTransform, mInverse;

  mTransform = mGvrc_obj * m_mTransform;
  mInverse = m_mInverse * mGobj_vrc;

  if ( eLOD > slidelodtSLF_BOUND )
    {
      RenderGeometry(mGvp_proj, mGproj_vrc, mGvrc_proj, mTransform, mInverse, 
		     pCBoundClip, plistLights, pCSLIDESurface, eShading, eLOD);
    }

  // Only RenderBound if we are cutting off at this point for LOD
  if ( eLOD == slidelodtSLF_BOUND )
    {
      RenderBound(mGvp_proj, mGproj_vrc, mTransform);
    }
}

VOID CSLIDEInstance::ModifySurfacePointer(CSLIDESurface* &pCSLIDESurface)
{
  if ( m_pCSLIDESurface != CSLIDESurface::GetSLF_INHERIT() )
    {
      pCSLIDESurface = m_pCSLIDESurface;
    }
}

//////////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 8: Handle Shading Flags
//
VOID CSLIDEInstance::ModifyShadingFlag(SLIDEShadingType &eShading)
{
}
//
// END  : ASSIGNMENT 8: Handle Shading Flags
//////////////////////////////////////////////////////////////////////////////

VOID CSLIDEInstance::ModifyLODFlag(SLIDELODType &eLOD)
{
  eLOD = (SLIDELODType)MIN((INT_32)eLOD, m_dyniLOD.GetValue());
}

CBound::BoundIntersectionType CSLIDEInstance::CheckBoundReject(const Matrix &/*m*/,
							    CBound *pCBoundClip)
{
  ASSERT( pCBoundClip != NULL );

  // No need to check bound because it is redundant, we will be doing an
  // equally effective check inside the CSLIDEObject

  return CBound::bitIntersect;
}

VOID CSLIDEInstance::RenderGeometry(const Matrix &mGvp_proj, 
				    const Matrix &mGproj_vrc, 
				    const Matrix &mGvrc_proj, 
				    const Matrix &mGvrc_obj, 
				    const Matrix &mGobj_vrc, 
				    CBound *pCBoundClip, 
				    CListInt *plistLights,
				    CSLIDESurface *pCSLIDESurface,
				    SLIDEShadingType eShading, 
				    SLIDELODType eLOD)
{
  //m_pCSLIDESurface->Push();
  {
    m_pCSLIDENode->Render(mGvp_proj, mGproj_vrc, mGvrc_proj, 
			  mGvrc_obj, mGobj_vrc, 
			  pCBoundClip, plistLights, pCSLIDESurface,
			  eShading, eLOD);
  }
  //m_pCSLIDESurface->Pop();
}

VOID CSLIDEInstance::RenderBound(const Matrix &mGvp_proj, 
				 const Matrix &mGproj_vrc, 
				 const Matrix &mGvrc_obj)
{
  // We do not store a bound at the instance level, so we must render
  // its child's bound transformed

  m_pCSLIDENode->RenderBound(mGvp_proj, mGproj_vrc, mGvrc_obj);
}

VOID CSLIDEInstance::WriteSLIDE(FILE *fpOutputFile)
{
  CListIterInt iter;
  CSLIDETransform *pCSLIDETransform;

  ASSERT( fpOutputFile != NULL );

  fprintf(fpOutputFile, "  %s %s\n", 
	  gcppcSLIDEReservedWords[sliderwtInstance], 
	  m_pcChildID);

  if ( m_pcID != NULL )
    {
      fprintf(fpOutputFile, "    %s %s\n",
	      gcppcSLIDEReservedWords[sliderwtID], 
	      m_pcID);
    } 
  if ( m_pcSurfaceID != gcppcSLIDEReservedWords[sliderwtSLF_INHERIT] )
    {
      fprintf(fpOutputFile, "    %s %s\n",
	      gcppcSLIDEReservedWords[sliderwtSurface], 
	      m_pcSurfaceID);
    } 
  if ( m_dyniShading.GetValue() != slideshtSLF_INHERIT )
    {
      fprintf(fpOutputFile, "    %s %s\n",
	      gcppcSLIDEReservedWords[sliderwtShading], 
	      gcppcSLIDEShadingReservedWords[m_dyniShading.GetValue()]);
    }
  if ( m_dyniLOD.GetValue() != slidelodtSLF_FULL )
    {
      fprintf(fpOutputFile, "    %s %s\n",
	      gcppcSLIDEReservedWords[sliderwtLOD], 
	      gcppcSLIDELODReservedWords[m_dyniLOD.GetValue()]);
    }

  iter.Init(&m_listCSLIDETransform);

  for ( pCSLIDETransform = (CSLIDETransform*)iter.PeekFirst(); 
	pCSLIDETransform != NULL;
	pCSLIDETransform = (CSLIDETransform*)iter.PeekNext() )
    {
      pCSLIDETransform->WriteSLIDE(fpOutputFile);      
    }

  fprintf(fpOutputFile, "  %s\n", 
	  gcppcSLIDEReservedWords[sliderwtEndInstance]);
}

UINT_32 CSLIDEInstance::GetTransformCount()
{
  return m_listCSLIDETransform.GetLength();
}

//////////////////////////////////////////////////////////////////////////////
// CSLIDEInstanceIter Class
//

CSLIDEInstanceIter::CSLIDEInstanceIter()
{
}

CSLIDEInstanceIter::~CSLIDEInstanceIter()
{
}

VOID CSLIDEInstanceIter::Init(CSLIDEInstance *pCSLIDEInstance)
{
  CSLIDEPrimitiveIter::Init(pCSLIDEInstance);

  m_iter.Init(&(pCSLIDEInstance->m_listCSLIDETransform));
}

VOID CSLIDEInstanceIter::Uninit()
{
  CSLIDEPrimitiveIter::Uninit();
}

CSLIDETransform *CSLIDEInstanceIter::PeekFirstCSLIDETransform()
{
  return (CSLIDETransform*)(m_iter.PeekFirst());
}

CSLIDETransform *CSLIDEInstanceIter::PeekNextCSLIDETransform()
{
  return (CSLIDETransform*)(m_iter.PeekNext());
}
