#include "slidelight.h"
#include "slidesurface.h"

BOOL CSLIDELight::m_bInfiniteViewer = TRUE;
Vector CSLIDELight::m_vInfiniteViewerDirection;

static const VECTOR4D gcvColor = { 1.0, 1.0, 1.0, 0.0 };
static const FLOAT gcfDeadDistance = (FLOAT)0.1;
static const FLOAT gcfFallOff = 1.0;
static const FLOAT gcfAngularFallOff = 1.0;

//////////////////////////////////////////////////////////////////////
// CSLIDELight Class
//

CSLIDELight::CSLIDELight()
{
  m_dyniType.SetValue(slideltSLF_AMBIENT);
  m_vColor.Set(gcvColor);
  m_dynfDeadDistance.SetValue(gcfDeadDistance);
  m_dynfFallOff.SetValue(gcfFallOff);
  m_dynfAngularFallOff.SetValue(gcfAngularFallOff);
}

CSLIDELight::~CSLIDELight()
{
}

VOID CSLIDELight::Init(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn, 
			CHAR *pcID)
{
  CSLIDEPrimitive::Init(pcFile, uiLine, uiColumn, pcID);

  m_dyniType.SetValue(slideltSLF_AMBIENT);
  m_vColor.Set(gcvColor);
  m_dynfDeadDistance.SetValue(gcfDeadDistance);
  m_dynfFallOff.SetValue(gcfFallOff);
  m_dynfAngularFallOff.SetValue(gcfAngularFallOff);
}

VOID CSLIDELight::Uninit()
{
  CSLIDEPrimitive::Uninit();
}

BOOL CSLIDELight::Initialized()
{
  return CSLIDEPrimitive::Initialized();
}

BOOL CSLIDELight::CheckSemantics(CSLIDEWorld *pCSLIDEWorld)
{
  return CSLIDEPrimitive::CheckSemantics(pCSLIDEWorld);
}

BOOL CSLIDELight::Valid()
{
  if ( !CSLIDEPrimitive::Valid() )
    {
      return FALSE;
    }

  return TRUE;
}

CSLIDECamera *CSLIDELight::FindCamera(CListIterInt *piterIDs, 
				       CListInt *plistCSLIDEInstance)
{
  ASSERT( piterIDs != NULL );
  ASSERT( plistCSLIDEInstance != NULL );

  HandleError("Bad %s path: %s \"%s\" encountered", 
	      gcppcSLIDEReservedWords[sliderwtCamera],
	      gcppcSLIDEReservedWords[sliderwtLight], GetID());

  return NULL;
}

CSLIDELight *CSLIDELight::FindLight(CListIterInt *piterIDs, 
				    CListInt *plistCSLIDEInstance)
{
  CHAR *pcID;
  CSLIDELight *pCSLIDELight;

  ASSERT( piterIDs != NULL );
  ASSERT( plistCSLIDEInstance != NULL );

  pCSLIDELight = this;

  pcID = (CHAR*)piterIDs->PeekNext();
  if ( pcID == NULL )
    {
      pcID = (CHAR*)piterIDs->PeekFirst();
      if ( pcID != GetID() )
	{
	  HandleError("Bad %s path: %s id \"%s\" not specified", 
		      gcppcSLIDEReservedWords[sliderwtLight],
		      gcppcSLIDEReservedWords[sliderwtLight], GetID());
	  pCSLIDELight = NULL;
	}
    }
  else if ( pcID != GetID() )
    {
      HandleError("Bad %s path: path id \"%s\" != %s id \"%s\"", 
		  gcppcSLIDEReservedWords[sliderwtLight], pcID,
		  gcppcSLIDEReservedWords[sliderwtLight], GetID());
      pCSLIDELight = NULL;
    }

  return pCSLIDELight;
}

VOID CSLIDELight::CheckDynamic()
{
  SetDynamic(m_dyniType.GetDynamic() ||
	     m_vColor.GetDynamic() ||
	     m_dynfDeadDistance.GetDynamic() ||
	     m_dynfFallOff.GetDynamic() ||
	     m_dynfAngularFallOff.GetDynamic());
}

VOID CSLIDELight::UpdateState()
{
  m_dyniType.Update();
  if ( !( 0 <= m_dyniType.GetValue() &&
	  m_dyniType.GetValue() < slideltSize ) )
    {
      HandleError("Illegal Light Type flag value:  %ld", 
		  m_dyniType.GetValue());
      m_dyniType.SetValue(slideltSLF_AMBIENT);
    }

  m_vColor.Update();
  m_dynfDeadDistance.Update();
  m_dynfFallOff.Update();
  m_dynfAngularFallOff.Update();
}

VOID CSLIDELight::Render(const Matrix &/*mGvp_proj*/, 
			 const Matrix &/*mGproj_vrc*/, 
			 const Matrix &/*mGvrc_proj*/, 
			 const Matrix &/*mGvrc_obj*/, 
			 const Matrix &/*mGobj_vrc*/, 
			 CBound * /*pCBoundClip*/, 
			 CListInt * /*plistLights*/,
			 CSLIDESurface * /*pCSLIDESurface*/,
			 SLIDEShadingType /*eShading*/, 
			 SLIDELODType /*eLOD*/)
{
}

//////////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 8: Implement the lighting calculation 
//

// pt and vNormal have both been transformed into the VRC coordinate system
// pCSLIDESurface is the surface property to use for the calculation
// mGlight_vrc and mGvrc_light are necessary for transforming the point, normal, 
//   and eye point or viewing direction into the light's coordinate system
Vector CSLIDELight::CalculateColor(const Point &pt, const Vector &vNormal, 
				   CSLIDESurface *pCSLIDESurface,
				   const Matrix &mGlight_vrc, 
				   const Matrix &mGvrc_light)
{
  // pt and vNormal are both in the light's local coordinate system
  // Remember to drop the W coord and normalize the normal

  SLIDELightType eType;
  Vector vColor;

  ASSERT( pCSLIDESurface != NULL );

  eType = (SLIDELightType)m_dyniType.GetValue();
  ASSERT( eType < slideltSize );

  // eType will be one of
  //    slideltSLF_AMBIENT
  //    slideltSLF_DIRECTIONAL
  //    slideltSLF_POINT
  //    slideltSLF_SPOT


  // Check whether the viewer is local or an infinite distance away
  //    GetInfiniteViewer() )
  // If infinite distance use the -DOP for the viewing direction
  //    m_vInfiniteViewerDirection


  // Actually do the correct lighting calculation instead of just 
  // returning the surface color
  vColor = pCSLIDESurface->GetColor();

  return vColor;
}
//
// END  : ASSIGNMENT 8: Implement the lighting calculation 
//////////////////////////////////////////////////////////////////////////////

VOID CSLIDELight::WriteSLIDE(FILE *fpOutputFile)
{
  ASSERT( fpOutputFile != NULL );
}

//////////////////////////////////////////////////////////////////////
// CSLIDELightPath Class
//

CSLIDELightPath::CSLIDELightPath()
{
  m_mGlight_vrc.SetIdentity();
  m_mGvrc_light.SetIdentity();
}

CSLIDELightPath::~CSLIDELightPath()
{
}

VOID CSLIDELightPath::Init(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn, 
			   CListInt *plistPathIDs, DynInt *pdyniOnOff)
{
  CSLIDEPath::Init(pcFile, uiLine, uiColumn, plistPathIDs);

  m_dyniOnOff.SetValue(slideoftSLF_ON);
  if ( pdyniOnOff != NULL )
    {
      m_dyniOnOff.Set(pdyniOnOff);
    }

  m_mGlight_vrc.SetIdentity();
  m_mGvrc_light.SetIdentity();
}

VOID CSLIDELightPath::Uninit()
{
  CSLIDEPath::Uninit();
}

CSLIDENode *CSLIDELightPath::FindNode(CSLIDENode *pRoot, CListIterInt *piterIDs, 
				       CListInt *plistCSLIDEInstance)
{
  ASSERT( pRoot != NULL );
  ASSERT( piterIDs != NULL );
  ASSERT( plistCSLIDEInstance != NULL );

  return pRoot->FindLight(piterIDs, plistCSLIDEInstance);
}

VOID CSLIDELightPath::CheckDynamic()
{
  CSLIDEPath::CheckDynamic();

  SetDynamic(GetDynamic() || m_dyniOnOff.GetDynamic());
}

VOID CSLIDELightPath::UpdateState()
{
  CSLIDEPath::UpdateState();

  m_dyniOnOff.Update();
  if ( !( 0 <= m_dyniOnOff.GetValue() &&
	  m_dyniOnOff.GetValue() < slideoftSize ) )
    {
      HandleError("Illegal OnOff flag value:  %ld", 
		  m_dyniOnOff.GetValue());
      m_dyniOnOff.SetValue(slideoftSLF_ON);
    }
}
