#ifndef __SLIDELIGHT_H__
#define __SLIDELIGHT_H__

#include "slidenode.h"
#include "slidepath.h"

//////////////////////////////////////////////////////////////////////
// CSLIDELight Class
//

class CSLIDELight : public CSLIDENode
{
//////////////////////////////////////////////////
// Enumerated Types
//

//////////////////////////////////////////////////
// Initialization
//
public:
  CSLIDELight();
  virtual ~CSLIDELight();
  virtual VOID Init(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn, 
		    CHAR *pcID);
  virtual VOID Uninit();

  virtual BOOL Initialized();

  virtual BOOL CheckSemantics(CSLIDEWorld *pCSLIDEWorld);

  virtual BOOL Valid();

//////////////////////////////////////////////////
// Geometric Computations
//
public:
  virtual CSLIDECamera *FindCamera(CListIterInt *piterIDs, 
				   CListInt *plistCSLIDEInstance);
  virtual CSLIDELight *FindLight(CListIterInt *piterIDs, 
				 CListInt *plistCSLIDEInstance);

protected:
  VOID CheckDynamic();
  VOID UpdateState();

//////////////////////////////////////////////////
// Rendering
//
public:
  virtual VOID Render(const Matrix &mGvp_proj, 
		      const Matrix &mGproj_vrc, const Matrix &mGvrc_proj, 
		      const Matrix &mGvrc_obj, const Matrix &mGobj_vrc, 
		      CBound *pCBoundClip, 
		      CListInt *plistLights,
		      CSLIDESurface *pCSLIDESurface,
		      SLIDEShadingType eShading, 
		      SLIDELODType eLOD);

//////////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 8: Implement the lighting calculation 
//
  virtual Vector CalculateColor(const Point &pt, const Vector &vNormal, 
				CSLIDESurface *pCSLIDESurface,
				const Matrix &mGlight_vrc, 
				const Matrix &mGvrc_light);
//
// END  : ASSIGNMENT 8: Implement the lighting calculation 
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////
// Output Functions
//
public:
  virtual VOID WriteSLIDE(FILE *fpOutputFile);

//////////////////////////////////////////////////
// Accessor Functions
//
public:
  inline VOID SetType(DynInt *pdyniType);
  inline VOID SetColor(DynVector *pdynvColor);
  inline VOID SetDeadDistance(DynFloat *pdynfDeadDistance);
  inline VOID SetFallOff(DynFloat *pdynfFallOff);
  inline VOID SetAngularFallOff(DynFloat *pdynfAngularFallOff);

  virtual inline CSLIDENode::NodeType GetNodeType();

//////////////////////////////////////////////////
// Class Accessor Functions
//
public:
//////////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 8: These are necessary if there is a Parallel Projection 
//
  static inline BOOL GetInfiniteViewer();
  static inline VOID SetInfiniteViewer(BOOL bInfiniteViewer);
  static inline VOID SetInfiniteViewerDirection(const Vector vV);

//////////////////////////////////////////////////
// Class Variables
//
protected:
  static BOOL m_bInfiniteViewer; // FALSE if using a Parallel Projection
  static Vector m_vInfiniteViewerDirection; // The viewing direction if Parallel Projection
//
// END  : ASSIGNMENT 8: These are necessary if there is a Parallel Projection 
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////
// Member Variables
//
protected:
  DynInt m_dyniType;
  DynVector m_vColor;
  DynFloat m_dynfDeadDistance;
  DynFloat m_dynfFallOff;
  DynFloat m_dynfAngularFallOff;
};

//////////////////////////////////////////////////////////////////////////////
// Inlined Functions
//

inline VOID CSLIDELight::SetType(DynInt *pdyniType)
{
  m_dyniType.Set(pdyniType);
}

inline VOID CSLIDELight::SetColor(DynVector *pdynvColor)
{
  m_vColor.SetAll(pdynvColor);
}

inline VOID CSLIDELight::SetDeadDistance(DynFloat *pdynfDeadDistance)
{
  m_dynfDeadDistance.Set(pdynfDeadDistance);
}

inline VOID CSLIDELight::SetFallOff(DynFloat *pdynfFallOff)
{
  m_dynfFallOff.Set(pdynfFallOff);
}

inline VOID CSLIDELight::SetAngularFallOff(DynFloat *pdynfAngularFallOff)
{
  m_dynfAngularFallOff.Set(pdynfAngularFallOff);
}

inline CSLIDENode::NodeType CSLIDELight::GetNodeType()
{
  return ntLight;
}

inline BOOL CSLIDELight::GetInfiniteViewer()
{
  return m_bInfiniteViewer;
}

inline VOID CSLIDELight::SetInfiniteViewer(BOOL bInfiniteViewer)
{
  m_bInfiniteViewer = bInfiniteViewer;
}

inline VOID CSLIDELight::SetInfiniteViewerDirection(const Vector vV)
{
  m_vInfiniteViewerDirection = vV;
  VERIFY( m_vInfiniteViewerDirection.Normalize() );
}

//////////////////////////////////////////////////////////////////////
// CSLIDELightPath Class
//

class CSLIDELightPath : public CSLIDEPath
{
//////////////////////////////////////////////////
// Initialization
//
public:
  CSLIDELightPath();
  virtual ~CSLIDELightPath();
  virtual VOID Init(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn, 
		    CListInt *plistPathIDs, DynInt *pdyniOnOff);
  virtual VOID Uninit();

  virtual CSLIDENode *FindNode(CSLIDENode *pRoot, CListIterInt *piterIDs, 
			       CListInt *plistCSLIDEInstance);

//////////////////////////////////////////////////
// Geometric Computations
//
public:

protected:
  virtual VOID CheckDynamic();
  virtual VOID UpdateState();

//////////////////////////////////////////////////
// Accessor Functions
//
public:
  inline CSLIDELight *GetLight();

  inline SLIDEOnOffType GetOnOff();

//////////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 8: These are necessary for the Eye to Light relationship
//
  inline const Matrix &GetGlight_vrc() const;
  inline Matrix &SetGlight_vrc();
  inline const Matrix &GetGvrc_light() const;
  inline Matrix &SetGvrc_light();
//
// END  : ASSIGNMENT 8: These are necessary for the Eye to Light relationship
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////
// Member Variables
//
protected:
  DynInt m_dyniOnOff;

  // Store the relationship of the camera to the eye to be passed down 
  // during the rendering pass.  We store it in the LightPath because 
  // they are unique, where as a single Light could be instanced multiple
  // times in the same render statement.
  Matrix m_mGlight_vrc; // VRC to Light coordinates
  Matrix m_mGvrc_light; // Light to VRC coordinates
};

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//

inline CSLIDELight *CSLIDELightPath::GetLight()
{
  return (CSLIDELight*)m_pNode;
}

inline SLIDEOnOffType CSLIDELightPath::GetOnOff()
{
  return (SLIDEOnOffType)m_dyniOnOff.GetValue();
}

inline const Matrix &CSLIDELightPath::GetGlight_vrc() const
{
  return m_mGlight_vrc;
}

inline Matrix &CSLIDELightPath::SetGlight_vrc()
{
  return m_mGlight_vrc;
}

inline const Matrix &CSLIDELightPath::GetGvrc_light() const
{
  return m_mGvrc_light;
}

inline Matrix &CSLIDELightPath::SetGvrc_light()
{
  return m_mGvrc_light;
}

#endif // __SLIDELIGHT_H__


