#include "slidenode.h"

#include "slideworld.h"
#include "slideface.h"

BOOL CSLIDENode::m_bRenderBound = FALSE;
BOOL CSLIDENode::m_bColorDynamicBounds = FALSE;
BOOL CSLIDENode::m_bColorChangedBounds = FALSE;
BOOL CSLIDENode::m_bCheckBoundChanged = TRUE;
BOOL CSLIDENode::m_bBoundReject = TRUE;

//////////////////////////////////////////////////////////////////////
// CSLIDENode Class
//

CSLIDENode::CSLIDENode()
{
}

CSLIDENode::~CSLIDENode()
{
}

VOID CSLIDENode::Init(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn, CHAR *pcID)
{
  CSLIDEPrimitive::Init(pcFile, uiLine, uiColumn, pcID);
}

VOID CSLIDENode::Uninit()
{
  CSLIDEPrimitive::Uninit();
}

BOOL CSLIDENode::Initialized()
{
  return CSLIDEPrimitive::Initialized();
}

BOOL CSLIDENode::CheckSemantics(CSLIDEWorld *pCSLIDEWorld)
{
  return CSLIDEPrimitive::CheckSemantics(pCSLIDEWorld);
}

BOOL CSLIDENode::Valid()
{
  return CSLIDEPrimitive::Valid();
}

VOID CSLIDENode::PreprocessGraph()
{
  SetBounded(FALSE);
  SetBoundDynamic(FALSE);
  SetBoundValid(TRUE);
  SetBoundChanged(FALSE);
}

VOID CSLIDENode::UpdateGraph()
{
}

VOID CSLIDENode::InsertIntoBound(const Matrix &/*m*/, CBound &/*bound*/)
{
  // This is the right thing to do for nodes with no bounds
  // like cameras and lights
}

VOID CSLIDENode::CalculateBound()
{
}

/*
CSLIDECamera *CSLIDENode::FindCamera(const Matrix &m, PATH )
{
}

CSLIDEPoint *CSLIDENode::FindPoint(const Matrix &m, PATH )
{
}
*/

VOID CSLIDENode::Render(const Matrix &mGvp_proj, 
			const Matrix &mGproj_vrc, const Matrix &mGvrc_proj, 
			const Matrix &mGvrc_obj, const Matrix &mGobj_vrc, 
			CBound *pCBoundClip, 
			CListInt *plistLights,
			CSLIDESurface *pCSLIDESurface,
			SLIDEShadingType eShading, 
			SLIDELODType eLOD)
{
  CBound::BoundIntersectionType eIntersect;

  ModifySurfacePointer(pCSLIDESurface);
  ModifyShadingFlag(eShading);
  ModifyLODFlag(eLOD);

  if ( eLOD == slidelodtSLF_OFF )
    {
      // This branch has been turned off by the LOD flags
      return;
    }

  // Check for Bound Rejection
  if ( GetBoundReject() && pCBoundClip != NULL )
    {
      // We have not been trivially accepted so we must check our bound
      // against the clip bound
      eIntersect = CheckBoundReject(mGproj_vrc * mGvrc_obj, pCBoundClip);

      if ( eIntersect == CBound::bitDisjoint )
	{
	  // We are entirely outside of the clipping region
	  // Trivially Reject
	  return;
	}
      else if ( eIntersect == CBound::bitEnclosed )
	{
	  // We are entirely outside of the clipping region
	  // Trivially Accept
	  pCBoundClip = NULL;
	}
    }

  if ( eLOD > slidelodtSLF_BOUND )
    {
      RenderGeometry(mGvp_proj, mGproj_vrc, mGvrc_proj, 
		     mGvrc_obj, mGobj_vrc, 
		     pCBoundClip, plistLights, pCSLIDESurface, 
		     eShading, eLOD);
    }

  if ( GetRenderBound() || eLOD == slidelodtSLF_BOUND )
    {
      RenderBound(mGvp_proj, mGproj_vrc, mGvrc_obj);
    }
}

VOID CSLIDENode::ModifySurfacePointer(CSLIDESurface* &/*pCSLIDESurface*/)
{
}

VOID CSLIDENode::ModifyShadingFlag(SLIDEShadingType &/*eShading*/)
{
}

VOID CSLIDENode::ModifyLODFlag(SLIDELODType &/*eLOD*/)
{
}

CBound::BoundIntersectionType CSLIDENode::CheckBoundReject(const Matrix &/*m*/,
						        CBound * /*pCBoundClip*/)
{
  // Default to not doing any rejections
  return CBound::bitIntersect;
}

VOID CSLIDENode::RenderGeometry(const Matrix &/*mGvp_proj*/, 
				const Matrix &/*mGproj_vrc*/, 
				const Matrix &/*mGvrc_proj*/, 
				const Matrix &/*mGvrc_obj*/, 
				const Matrix &/*mGobj_vrc*/, 
				CBound * /*pCBoundClip*/, 
				CListInt * /*plistLights*/,
				CSLIDESurface * /*pCSLIDESurface*/,
				SLIDEShadingType /*eShading*/, 
				SLIDELODType /*eLOD*/)
{
}

VOID CSLIDENode::RenderBound(const Matrix &/*mGvp_proj*/, 
			     const Matrix &/*mGproj_vrc*/, 
			     const Matrix &/*mGvrc_obj*/)
{
}


