#ifndef __SLIDERENDER_H__
#define __SLIDERENDER_H__

#include "slideprimitive.h"

class CSLIDEWorld;
class CSLIDECameraPath;
class CSLIDELightPath;
class CSLIDENode;

class CSLIDETclVariables;

//////////////////////////////////////////////////////////////////////
// CSLIDERender Class
//

class CSLIDERender : public CSLIDEPrimitive
{
//////////////////////////////////////////////////
// Friends
//
public:

//////////////////////////////////////////////////
// Enumerated Types
//

//////////////////////////////////////////////////
// Initialization
//
public:
  CSLIDERender();
  virtual ~CSLIDERender();
  virtual VOID Init(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn, 
		    CHAR *pcViewportID, CSLIDECameraPath *pCSLIDECameraPath,
		    CHAR *pcWorldRootID);
  virtual VOID Uninit();

  virtual BOOL Initialized();

  virtual inline VOID SetTclTag(CHAR *pcTclTag);
  virtual inline VOID SetLOD(DynInt *pdyniLOD);
  virtual inline VOID AddLightPath(CSLIDELightPath *pCSLIDELightPath);

  virtual BOOL CheckSemantics(CSLIDEWorld *pCSLIDEWorld);

  virtual BOOL Valid();

public:  

//////////////////////////////////////////////////
// Iteration Functions
//
public:

//////////////////////////////////////////////////
// Geometric Computations
//
public:
  virtual VOID PreprocessGraph();
  virtual VOID UpdateGraph();

protected:
  virtual VOID CheckDynamic();
  virtual VOID UpdateState();

//////////////////////////////////////////////////
// Input Functions
//
public:
  VOID HandleReshape(const Vector &vSize);
  BOOL HandleMouseButton(const Vector &vSize,
			 INT_32 iButton, INT_32 iState, 
			 const Vector &vClick);
  BOOL HandleMouseMotion(const Vector &vSize,
			 const Vector &vClick);

//////////////////////////////////////////////////
// Rendering
//
public:
//////////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 8: Deal with lighting and shading
//
  virtual VOID Render(const Vector &vOrigin, const Vector &vSize);
//
// END  : ASSIGNMENT 8: Deal with lighting and shading
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////
// Output Functions
//
public:
  virtual VOID WriteSLIDE(FILE *fpOutputFile);

//////////////////////////////////////////////////
// Accessor Functions
//
public:
  static inline BOOL GetRenderClipBound();
  static inline VOID SetRenderClipBound(BOOL bRenderClipBound);
  static inline BOOL GetLightingEnabled();
  static inline VOID SetLightingEnabled(BOOL bLightingEnabled);
  static inline BOOL GetCrystalBallEnabled();
  static inline VOID SetCrystalBallEnabled(BOOL bCrystalBallEnabled);
  static inline BOOL GetRotateClippedEnabled();
  static inline VOID SetRotateClippedEnabled(BOOL bRotateClippedEnabled);

//////////////////////////////////////////////////
// Class Variables
//
protected:
  static BOOL m_bRenderClipBound;
  static BOOL m_bLightingEnabled;
  static BOOL m_bCrystalBallEnabled;
  static BOOL m_bRotateClippedEnabled;

  static UINT_32 m_uiCount;
  static Vector m_vProjMin;
  static Vector m_vProjMax;
  static CBound m_boundClip;

  static CListInt *m_plistCSLIDETclVariables;

//////////////////////////////////////////////////
// Member Variables
//
protected:
  CHAR *m_pcTclTag;
  CSLIDETclVariables *m_pCSLIDETclVariables;

  CHAR *m_pcViewportID;

  CSLIDECameraPath *m_pCSLIDECameraPath;

  CHAR *m_pcWorldRootID;
  CSLIDENode *m_pWorldRoot;

  DynInt m_dyniLOD;

  CListInt m_listCSLIDELightPath;

  // Crystal Ball Variables
  Matrix m_mBall;
  Matrix m_mBallInverse;

  INT_32 m_iButtonCurr;
  Vector m_vStart;
  Vector m_vLatest;
  FLOAT m_fScaleLast;
};

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//

inline VOID CSLIDERender::SetTclTag(CHAR *pcTclTag)
{
  m_pcTclTag = pcTclTag;
}

inline VOID CSLIDERender::SetLOD(DynInt *pdyniLOD)
{
  m_dyniLOD.Set(pdyniLOD);
}

inline VOID CSLIDERender::AddLightPath(CSLIDELightPath *pCSLIDELightPath)
{
  ASSERT( pCSLIDELightPath != NULL );
  m_listCSLIDELightPath.InsertLast(pCSLIDELightPath, 0);
}

inline BOOL CSLIDERender::GetRenderClipBound()
{
  return m_bRenderClipBound;
}

inline VOID CSLIDERender::SetRenderClipBound(BOOL bRenderClipBound)
{
  m_bRenderClipBound = bRenderClipBound;
}

inline BOOL CSLIDERender::GetLightingEnabled()
{
  return m_bLightingEnabled;
}

inline VOID CSLIDERender::SetLightingEnabled(BOOL bLightingEnabled)
{
  m_bLightingEnabled = bLightingEnabled;
}

inline BOOL CSLIDERender::GetCrystalBallEnabled()
{
  return m_bCrystalBallEnabled;
}

inline VOID CSLIDERender::SetCrystalBallEnabled(BOOL bCrystalBallEnabled)
{
  m_bCrystalBallEnabled = bCrystalBallEnabled;
}

inline BOOL CSLIDERender::GetRotateClippedEnabled()
{
  return m_bRotateClippedEnabled;
}

inline VOID CSLIDERender::SetRotateClippedEnabled(BOOL bRotateClippedEnabled)
{
  m_bRotateClippedEnabled = bRotateClippedEnabled;
}

//////////////////////////////////////////////////////////////////////
// CSLIDETclVariables Class
//

class CSLIDETclVariables
{
//////////////////////////////////////////////////
// Initialization
//
public:
  CSLIDETclVariables();
  virtual ~CSLIDETclVariables();
  virtual VOID Init(CHAR *pcID);
  virtual VOID Uninit();

  virtual BOOL Initialized();
  virtual BOOL Valid();

//////////////////////////////////////////////////
// Accessor Functions
//
public:
  inline CHAR *GetID() const;

  inline FLOAT GetWidth() const;
  VOID SetWidth(FLOAT fWidth);
  inline FLOAT GetHeight() const;
  VOID SetHeight(FLOAT fHeight);
  inline FLOAT GetMouseX() const;
  VOID SetMouseX(FLOAT fMouseX);
  inline FLOAT GetMouseY() const;
  VOID SetMouseY(FLOAT fMouseY);
  inline const Quater &GetRotate() const;
  VOID SetRotate(const Quater &q);
  inline FLOAT GetScale() const;
  VOID SetScale(FLOAT fScale);

//////////////////////////////////////////////////
// Member Variables
//
protected:
  CHAR *m_pcID;

  // Window Variables
  FLOAT m_fWidth;
  FLOAT m_fHeight;
  FLOAT m_fMouseX;
  FLOAT m_fMouseY;

  // Crystal Ball Variables
  Quater m_qRotate;
  FLOAT m_fScale;
};

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//

inline CHAR *CSLIDETclVariables::GetID() const
{
  return m_pcID;
}

inline FLOAT CSLIDETclVariables::GetWidth() const
{
  return m_fWidth;
}

inline FLOAT CSLIDETclVariables::GetHeight() const
{
  return m_fHeight;
}

inline FLOAT CSLIDETclVariables::GetMouseX() const
{
  return m_fMouseX;
}

inline FLOAT CSLIDETclVariables::GetMouseY() const
{
  return m_fMouseY;
}

inline const Quater &CSLIDETclVariables::GetRotate() const
{
  return m_qRotate;
}

inline FLOAT CSLIDETclVariables::GetScale() const
{
  return m_fScale;
}

#endif // __SLIDERENDER_H__
