#include "vector.h"
#include "point.h"

Vector::Vector()
{
  Null();
}

Vector::Vector(FLOAT x, FLOAT y, FLOAT z, FLOAT w/* = 0.0f*/)
{
  m_v[0] = x;
  m_v[1] = y;
  m_v[2] = z;
  m_v[3] = w;
}

Vector::~Vector()
{
}

VOID Vector::Update()
{
}

FLOAT Vector::Magnitude() const
{
  return (FLOAT)sqrt(Dot(*this));
}

VOID Vector::Null()
{
  memset(m_v, 0, sizeof(VECTOR4D));
}

BOOL Vector::Normalize()
{
  FLOAT fMag;

  fMag = Magnitude();

  if ( fMag <= 0.0f )
    {
      return FALSE;
    }

  m_v[0] = m_v[0] / fMag;
  m_v[1] = m_v[1] / fMag;
  m_v[2] = m_v[2] / fMag;
  m_v[3] = m_v[3] / fMag;

  return TRUE;
}

VOID Vector::CompletePlaneEquation(const Point &pt)
{
  Vector v;

  m_v[3] = 0.0;
  Normalize();
  v = pt;
  m_v[3] = -(this->Dot(v));
}

Vector &Vector::operator=(const Vector &v)
{
  m_v[0] = v(0);
  m_v[1] = v(1);
  m_v[2] = v(2);
  m_v[3] = v(3);

  return *this;
}

Vector &Vector::operator=(const Point &pt)
{
  m_v[0] = pt(0);
  m_v[1] = pt(1);
  m_v[2] = pt(2);
  m_v[3] = pt(3);

  return *this;
}

Vector &Vector::operator+=(const Vector &v)
{
  m_v[0] = m_v[0] + v(0);
  m_v[1] = m_v[1] + v(1);
  m_v[2] = m_v[2] + v(2);
  m_v[3] = m_v[3] + v(3);

  return *this;
}

Vector &Vector::operator-=(const Vector &v)
{
  m_v[0] = m_v[0] - v(0);
  m_v[1] = m_v[1] - v(1);
  m_v[2] = m_v[2] - v(2);
  m_v[3] = m_v[3] - v(3);

  return *this;
}

Vector &Vector::operator*=(const FLOAT f)
{
  m_v[0] = m_v[0] * f;
  m_v[1] = m_v[1] * f;
  m_v[2] = m_v[2] * f;
  m_v[3] = m_v[3] * f;

  return *this;
}

Vector &Vector::operator/=(const FLOAT f)
{
  m_v[0] = m_v[0] / f;
  m_v[1] = m_v[1] / f;
  m_v[2] = m_v[2] / f;
  m_v[3] = m_v[3] / f;

  return *this;
}

BOOL Vector::operator==(const Vector &v) const
{
  FLOAT fDot, fDotV, fVDotV;

  fDot = this->Dot(*this);
  fDotV = this->Dot(v);
  fVDotV = v.Dot(v);

  return ((fDot > (fDotV - EPSILON)) && (fDot < (fDotV + EPSILON))) &&
         ((fDot > (fVDotV - EPSILON)) && (fDot < (fVDotV + EPSILON)));
}

BOOL Vector::operator!=(const Vector &v) const
{
  return !(*this == v);
}

FLOAT Vector::Dot(const Vector &v) const
{
  return (((*this)(0)*v(0)) + ((*this)(1)*v(1)) + 
          ((*this)(2)*v(2)) + ((*this)(3)*v(3))); 
}

VOID Vector::Display() const
{
  UINT_32 i;

  printf("< ");
  for ( i = 0; i < 4; i++)
    {
      printf("%3.3f ", (*this)(i));
    }
  printf(">\n");
}

DOUBLE AngleBetween(const Vector &v1, const Vector &v2)
{
  Vector w;
  DOUBLE dTheta;
  FLOAT fMag1, fMag2;

  fMag1 = v1.Magnitude();
  fMag2 = v2.Magnitude();

  if ( fMag1 == 0.0f || fMag2 == 0.0f )
    {
      return 0.0f;
    }

  dTheta = (DOUBLE)((v1.Dot(v2))/(fMag1*fMag2));        

  if ( dTheta > 1.0f )
    {
      dTheta = 1.0f;
    }

  if ( dTheta < -1.0f )
    {
      dTheta = -1.0f;
    }

  dTheta = acos(dTheta);
  /*
  w = v1.Cross(v2);

  if ( w(3) < 0 )
    {
      return -dTheta;
    }
    */
  return dTheta;
}

//////////////////////////////////////////////////////////////////////
// DynVector Class
//

DynVector::DynVector()
{
  Null();

  m_ppcDynExpr[0] = NULL;
  m_ppcDynExpr[1] = NULL;
  m_ppcDynExpr[2] = NULL;
  m_ppcDynExpr[3] = NULL;
}

DynVector::DynVector(FLOAT x, FLOAT y, FLOAT z, FLOAT w/* = 0.0f*/)
{
  m_v[0] = x;
  m_v[1] = y;
  m_v[2] = z;
  m_v[3] = w;

  m_ppcDynExpr[0] = NULL;
  m_ppcDynExpr[1] = NULL;
  m_ppcDynExpr[2] = NULL;
  m_ppcDynExpr[3] = NULL;
}

DynVector::~DynVector()
{
  m_ppcDynExpr[0] = NULL;
  m_ppcDynExpr[1] = NULL;
  m_ppcDynExpr[2] = NULL;
  m_ppcDynExpr[3] = NULL;
}

VOID DynVector::Update()
{
  DynEval(m_v, m_ppcDynExpr, 4);
}

DynVector &DynVector::operator=(const Vector &v)
{
  m_v[0] = v(0);
  m_v[1] = v(1);
  m_v[2] = v(2);
  m_v[3] = v(3);

  return *this;
}
